@@ -1,4 +1,4 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """Rosbag1 to Rosbag2 Converter."""
 
@@ -47,6 +47,13 @@ def command(
         },
     ],
     dst: Annotated[Path, {'flags': ['--dst']}],
+    dst_storage: Annotated[
+        str,
+        {
+            'flags': ['--dst-storage'],
+            'choices': ['sqlite3', 'mcap'],
+        },
+    ] = 'sqlite3',
     dst_version: Annotated[int, {'flags': ['--dst-version']}] = 8,
     compress: Annotated[
         str,
@@ -59,7 +66,7 @@ def command(
         str,
         {
             'flags': ['--compress-mode'],
-            'choices': ['file', 'message'],
+            'choices': ['file', 'message', 'storage'],
         },
     ] = 'file',
     src_typestore: Annotated[
@@ -170,7 +177,7 @@ def command(
         Convert bag from rosbag1 to rosbag2, using per file compression for destination:
             rosbags-convert --src example.bag --dst ros2_bagdir --compress zstd
 
-        Convert bag from rosbag1 to rosbag2, upgrate types to iron:
+        Convert bag from rosbag1 to rosbag2, upgrade types to iron:
             rosbags-convert --src example.bag --dst ros2_bagdir --dst-typestore ros2_iron
 
         Convert bag from legacy rosbag2 (with humble types) to rosbag1:
@@ -182,6 +189,7 @@ def command(
     Args:
         srcs: Rosbag files to read from.
         dst: Destination path to write rosbag to.
+        dst_storage: Destination file storage backend.
         dst_version: Destination file format version.
         compress: Compression algorithm.
             Rosbag1 supports 'bz2' or 'lz4'.
@@ -250,6 +258,7 @@ def command(
         convert(
             srcs,
             dst,
+            dst_storage,
             dst_version,
             compress if compress != 'none' else None,
             compress_mode,
