@@ -1,4 +1,4 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """Rosbag1 writer."""
 
@@ -18,7 +18,12 @@ from typing import TYPE_CHECKING
 
 from lz4.frame import compress as lz4_compress  # type: ignore[import-untyped]
 
-from rosbags.interfaces import Connection, ConnectionExtRosbag1
+from rosbags.interfaces import (
+    Connection,
+    ConnectionExtRosbag1,
+    MessageDefinition,
+    MessageDefinitionFormat,
+)
 from rosbags.typesys import Stores, get_typestore
 from rosbags.typesys.msg import denormalize_msgtype
 
@@ -164,7 +169,7 @@ class Writer:
             path: Filesystem path to bag.
 
         Raises:
-            WriterError: Target path exisits already, Writer can only create new rosbags.
+            WriterError: Target path exists already, Writer can only create new rosbags.
 
         """
         path = Path(path)
@@ -242,7 +247,7 @@ class Writer:
             topic: Topic name.
             msgtype: Message type.
             typestore: Typestore.
-            msgdef: Message definiton.
+            msgdef: Message definition.
             md5sum: Message hash.
             callerid: Caller id.
             latching: Latching information.
@@ -274,7 +279,7 @@ class Writer:
             len(self.connections),
             topic,
             msgtype,
-            msgdef,
+            MessageDefinition(MessageDefinitionFormat.MSG, msgdef),
             md5sum,
             -1,
             ConnectionExtRosbag1(callerid, latching),
@@ -338,7 +343,7 @@ class Writer:
         header.set_string('topic', connection.topic)
         header.set_string('type', denormalize_msgtype(connection.msgtype))
         header.set_string('md5sum', connection.digest)
-        header.set_string('message_definition', connection.msgdef)
+        header.set_string('message_definition', connection.msgdef.data)
         assert isinstance(connection.ext, ConnectionExtRosbag1)
         if connection.ext.callerid is not None:
             header.set_string('callerid', connection.ext.callerid)
