@@ -1,4 +1,4 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """Rosbag2 errors."""
 
@@ -9,3 +9,9 @@ class ReaderError(Exception):
     """Reader Error."""
 
     args: tuple[str]
+
+
+class WriterError(Exception):
+    """Writer Error."""
+
+    args: tuple[str]
