@@ -1,9 +1,9 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """MSG Parser.
 
 Grammar, parse tree visitor and conversion functions for message definitions in
-`MSG`_ format. It also supports concatened message definitions as found in
+`MSG`_ format. It also supports concatenated message definitions as found in
 Rosbag1 connection information.
 
 .. _MSG: http://wiki.ros.org/msg
@@ -159,7 +159,7 @@ def normalize_msgtype(name: str) -> str:
     return str(path)
 
 
-def normalize_fieldtype(typename: str, idx: int, field: FieldDesc) -> FieldDesc:
+def normalize_fieldtype(typename: str, field: FieldDesc) -> FieldDesc:
     """Normalize field typename.
 
     Args:
@@ -184,7 +184,7 @@ def normalize_fieldtype(typename: str, idx: int, field: FieldDesc) -> FieldDesc:
     assert ifield[0] == Nodetype.NAME
 
     name = ifield[1]
-    if name == 'Header' and not idx:
+    if name == 'Header':
         name = 'std_msgs/msg/Header'
     elif '/' not in name:
         name = str(Path(typename).parent / name)
@@ -256,7 +256,7 @@ class VisitorMSG(Visitor):
                 else:
                     assert item[0] == Node.FIELD
                     fields.append(
-                        (item[1][0], normalize_fieldtype(name, len(fields), item[1][1])),
+                        (item[1][0], normalize_fieldtype(name, item[1][1])),
                     )
 
             res[name] = consts, fields
@@ -361,7 +361,7 @@ def get_types_from_msg(text: str, name: str) -> Typesdict:
     """Get type from msg message definition.
 
     Args:
-        text: Message definiton.
+        text: Message definition.
         name: Message typename.
 
     Returns:
