@@ -1,4 +1,4 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """Concrete store implementations."""
 
@@ -14,7 +14,7 @@ class Stores(Enum):
     EMPTY = 'empty'
     """Only builtin messages (Duration, Time)."""
     LATEST = 'latest'
-    """Alias for lastest ROS2 LTS."""
+    """Alias for latest ROS2 LTS."""
     ROS1_NOETIC = 'ros1_noetic'
     """Noetic Ninjemys."""
     ROS2_DASHING = 'ros2_dashing'
@@ -29,12 +29,14 @@ class Stores(Enum):
     """Humble Hawksbill."""
     ROS2_IRON = 'ros2_iron'
     """Iron Irwini."""
+    ROS2_JAZZY = 'ros2_jazzy'
+    """Jazzy Jalisco."""
 
 
 def get_typestore(name: Stores) -> Typestore:
     """Get typestore by name."""
     if name == Stores.LATEST:
-        name = Stores.ROS2_FOXY
+        name = Stores.ROS2_JAZZY
 
     mod = import_module(f'.{name.value}', __name__)
     return Typestore(mod)
