@@ -1,4 +1,4 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """Rosbag Converter Tests."""
 
@@ -22,7 +22,14 @@ from rosbags.convert.converter import (
     migrate_message,
 )
 from rosbags.highlevel import AnyReaderError
-from rosbags.interfaces import Connection, ConnectionExtRosbag1, ConnectionExtRosbag2, Qos
+from rosbags.interfaces import (
+    Connection,
+    ConnectionExtRosbag1,
+    ConnectionExtRosbag2,
+    MessageDefinition,
+    MessageDefinitionFormat,
+    Qos,
+)
 from rosbags.rosbag1 import (
     Writer as Writer1,
     WriterError as WriterError1,
@@ -62,7 +69,7 @@ def test_convert_reader_errors(tmp_path: Path) -> None:
         patch('rosbags.convert.converter.AnyReader', side_effect=AnyReaderError('exc')),
         pytest.raises(ConverterError, match='Reading source bag: exc'),
     ):
-        convert([], tmp_path / 'foo', None, None, 'file', None, None, (), (), (), ())
+        convert([], tmp_path / 'foo', 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
 
 
 def test_convert_writer_errors(tmp_path: Path) -> None:
@@ -71,13 +78,13 @@ def test_convert_writer_errors(tmp_path: Path) -> None:
         patch('rosbags.convert.converter.Writer2', side_effect=WriterError2('exc')),
         pytest.raises(ConverterError, match='Writing destination bag: exc'),
     ):
-        convert([], tmp_path / 'foo', None, None, 'file', None, None, (), (), (), ())
+        convert([], tmp_path / 'foo', 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
 
     with (
         patch('rosbags.convert.converter.Writer1', side_effect=WriterError1('exc')),
         pytest.raises(ConverterError, match='Writing destination bag: exc'),
     ):
-        convert([], tmp_path / 'foo.bag', None, None, 'file', None, None, (), (), (), ())
+        convert([], tmp_path / 'foo.bag', 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
 
 
 def test_convert_forwards_exceptions(tmp_path: Path) -> None:
@@ -86,7 +93,38 @@ def test_convert_forwards_exceptions(tmp_path: Path) -> None:
         patch('rosbags.convert.converter.AnyReader', side_effect=KeyError('exc')),
         pytest.raises(ConverterError, match="Converting rosbag: KeyError\\('exc'\\)"),
     ):
-        convert([], tmp_path / 'foo', None, None, 'file', None, None, (), (), (), ())
+        convert([], tmp_path / 'foo', 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
+
+
+def test_convert_enables_compression(tmp_path: Path) -> None:
+    """Test convert forwards exceptions."""
+    with (
+        patch('rosbags.convert.converter.AnyReader'),
+        patch('rosbags.convert.converter.Writer1') as writer,
+    ):
+        convert([], tmp_path / 'foo.bag', 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
+    writer.return_value.set_compression.assert_not_called()
+
+    with (
+        patch('rosbags.convert.converter.AnyReader'),
+        patch('rosbags.convert.converter.Writer1') as writer,
+    ):
+        convert([], tmp_path / 'foo.bag', 'sqlite3', 8, 'bz2', 'file', None, None, (), (), (), ())
+    writer.return_value.set_compression.assert_called()
+
+    with (
+        patch('rosbags.convert.converter.AnyReader'),
+        patch('rosbags.convert.converter.Writer2') as writer,
+    ):
+        convert([], tmp_path / 'foo', 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
+    writer.return_value.set_compression.assert_not_called()
+
+    with (
+        patch('rosbags.convert.converter.AnyReader'),
+        patch('rosbags.convert.converter.Writer2') as writer,
+    ):
+        convert([], tmp_path / 'foo', 'sqlite3', 8, 'zstd', 'file', None, None, (), (), (), ())
+    writer.return_value.set_compression.assert_called()
 
 
 def test_convert_connection_filtering(tmp_path: Path) -> None:
@@ -97,7 +135,7 @@ def test_convert_connection_filtering(tmp_path: Path) -> None:
         patch('rosbags.convert.converter.create_connections_converters') as ccc,
     ):
         reader.return_value.__enter__.return_value.connections = []
-        convert([], tmp_path, None, None, 'file', None, None, (), (), (), ())
+        convert([], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
     ccc.assert_not_called()
 
     with (
@@ -113,27 +151,31 @@ def test_convert_connection_filtering(tmp_path: Path) -> None:
         conn.msgtype = 'bar'
         reader.return_value.__enter__.return_value.connections = [conn]
         with pytest.raises(ConverterError):
-            convert([], tmp_path, None, None, 'file', None, None, (), (), (), ())
+            convert([], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
         ccc.reset_mock()
 
         with pytest.raises(ConverterError):
-            convert([], tmp_path, None, None, 'file', None, None, (), ('foo'), (), ('unknown'))
+            convert(
+                [], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), ('foo'), (), ('unknown')
+            )
         ccc.reset_mock()
 
         with pytest.raises(ConverterError):
-            convert([], tmp_path, None, None, 'file', None, None, (), ('unknown'), (), ('bar'))
+            convert(
+                [], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), ('unknown'), (), ('bar')
+            )
         ccc.reset_mock()
 
-        convert([], tmp_path, None, None, 'file', None, None, ('foo'), (), (), ())
+        convert([], tmp_path, 'sqlite3', 8, None, 'file', None, None, ('foo'), (), (), ())
         ccc.assert_not_called()
 
-        convert([], tmp_path, None, None, 'file', None, None, (), (), ('bar'), ())
+        convert([], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), (), ('bar'), ())
         ccc.assert_not_called()
 
-        convert([], tmp_path, None, None, 'file', None, None, (), ('unknown'), (), ())
+        convert([], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), ('unknown'), (), ())
         ccc.assert_not_called()
 
-        convert([], tmp_path, None, None, 'file', None, None, (), (), (), ('unknown'))
+        convert([], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), (), (), ('unknown'))
         ccc.assert_not_called()
 
 
@@ -159,7 +201,7 @@ def test_convert_applies_transforms(tmp_path: Path) -> None:
             {'bar': lambda x: x * 2},  # pyright: ignore[reportUnknownLambdaType]
         ]
 
-        convert([], tmp_path, None, None, 'file', None, None, (), (), (), ())
+        convert([], tmp_path, 'sqlite3', 8, None, 'file', None, None, (), (), (), ())
 
         writer.return_value.write.assert_called_with(666, 1, 4)
 
@@ -171,7 +213,7 @@ def test_connection_params_are_converted() -> None:
             1,
             '/t1',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1(None, latching=1),
@@ -181,7 +223,7 @@ def test_connection_params_are_converted() -> None:
             2,
             '/t2',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1('caller', latching=0),
@@ -243,7 +285,7 @@ def test_connection_params_are_converted() -> None:
             1,
             '/t1',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag2('cdr', LATCH),
@@ -253,7 +295,7 @@ def test_connection_params_are_converted() -> None:
             2,
             '/t2',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag2('cdr', []),
@@ -318,7 +360,7 @@ def test_destination_typestore_gets_created() -> None:
             1,
             '/t1',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1('', latching=True),
@@ -421,7 +463,7 @@ def test_connection_deduplication() -> None:
             1,
             '/t1',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1('', latching=True),
@@ -431,7 +473,7 @@ def test_connection_deduplication() -> None:
             2,
             '/t1',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1('', latching=True),
@@ -441,7 +483,7 @@ def test_connection_deduplication() -> None:
             3,
             '/t1',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1('', latching=False),
@@ -451,7 +493,7 @@ def test_connection_deduplication() -> None:
             4,
             '/t1',
             'std_msgs/msg/Int8',
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1('foo', latching=False),
@@ -475,7 +517,7 @@ def test_connection_deduplication() -> None:
             len(writer.connections),
             name,
             typ,
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag1(callerid, latching),
@@ -515,7 +557,7 @@ def test_connection_deduplication() -> None:
             len(writer.connections),
             name,
             typ,
-            '',
+            MessageDefinition(MessageDefinitionFormat.NONE, ''),
             '',
             0,
             ConnectionExtRosbag2(serialization_format, offered_qos_profiles),
