@@ -1,4 +1,4 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """Reader Tests."""
 
@@ -16,8 +16,9 @@ import pytest
 import zstandard
 from ruamel.yaml import YAML
 
-from rosbags.rosbag2 import Reader, ReaderError, Writer
+from rosbags.rosbag2 import Reader, ReaderError
 from rosbags.rosbag2.metadata import Metadata
+from rosbags.rosbag2.storage_sqlite3 import Sqlite3Writer
 
 if TYPE_CHECKING:
     from collections.abc import Iterable
@@ -89,17 +90,17 @@ rosbag2_bagfile_information:
 
 @pytest.fixture
 def empty_bag(tmp_path: Path) -> Path:
-    """Manually contruct empty bag."""
+    """Manually construct empty bag."""
     _ = (tmp_path / 'metadata.yaml').write_text(METADATA_EMPTY)
     dbpath = tmp_path / 'db.db3'
     dbh = sqlite3.connect(dbpath)
-    _ = dbh.executescript(Writer.SQLITE_SCHEMA)
+    _ = dbh.executescript(Sqlite3Writer.SQLITE_SCHEMA)
     return tmp_path
 
 
 @pytest.fixture(params=['none', 'file', 'message'])
 def bag_sqlite3(request: SubRequest, tmp_path: Path) -> Path:
-    """Manually contruct sqlite3 bag."""
+    """Manually construct sqlite3 bag."""
     param: str = request.param  # pyright: ignore[reportAny]
     _ = (tmp_path / 'metadata.yaml').write_text(
         METADATA.format(
@@ -113,7 +114,7 @@ def bag_sqlite3(request: SubRequest, tmp_path: Path) -> Path:
 
     dbpath = tmp_path / 'db.db3'
     dbh = sqlite3.connect(dbpath)
-    _ = dbh.executescript(Writer.SQLITE_SCHEMA)
+    _ = dbh.executescript(Sqlite3Writer.SQLITE_SCHEMA)
 
     cur = dbh.cursor()
     _ = cur.execute(
@@ -192,6 +193,7 @@ def test_empty_bag(empty_bag: Path) -> None:
         assert reader.end_time == 0
         assert reader.duration == 0
         assert not list(reader.messages())
+        assert reader.custom_data
         assert reader.custom_data['key1'] == 'value1'
         assert reader.custom_data['key2'] == 'value2'
         assert reader.ros_distro == 'rosbags'
@@ -442,7 +444,7 @@ CHANNELS = [
     params=['unindexed', 'partially_indexed', 'indexed', 'chunked_unindexed', 'chunked_indexed'],
 )
 def bag_mcap(request: SubRequest, tmp_path: Path) -> Path:
-    """Manually contruct mcap bag."""
+    """Manually construct mcap bag."""
     param: str = request.param  # pyright: ignore[reportAny]
     _ = (tmp_path / 'metadata.yaml').write_text(
         METADATA.format(
